clear all;
close all;
clc;

x0 = [0;0;0;0];
u = 0.01;
T = 4;

%% Nonlinear simulation using the RK4 integrator
N = 80;
Ts = T/N;
x_rk4 = x0;
input.Ts = Ts;
input.nSteps = 2;
input.u = u;
t_rk4 = [0:N].*Ts;
for i = 1:N
    input.x = x_rk4(:,end);
    output = RK4_integrator( @ode, input );
    x_rk4(:,end+1) = output.value;
    visualize(t_rk4, x_rk4, 0.8);
    pause(Ts/2)
end

% Comparison with ode45
[t_ode45,x_ode45] = ode45(@(t,x) ode(t,x,u), [0 T], x0);

%% Plot results:
figure;
set(gcf,'Color',[1 1 1])
subplot(221);
plot(t_ode45,x_ode45(:,1),'--g'); hold on
plot(t_rk4,x_rk4(1,:),'rx','MarkerSize',10);
xlabel('time(s)')
ylabel('p')
legend('ode45', 'RK4');
grid on;

subplot(222);
plot(t_ode45,x_ode45(:,2),'--g'); hold on
plot(t_rk4,x_rk4(2,:),'rx','MarkerSize',10);
xlabel('time(s)')
ylabel('\theta')
legend('ode45', 'RK4');
grid on;

subplot(223);
plot(t_ode45,x_ode45(:,3),'--g'); hold on
plot(t_rk4,x_rk4(3,:),'rx','MarkerSize',10);
xlabel('time(s)')
ylabel('v')
legend('ode45', 'RK4');
grid on;

subplot(224);
plot(t_ode45,x_ode45(:,4),'--g'); hold on
plot(t_rk4,x_rk4(4,:),'rx','MarkerSize',10);
xlabel('time(s)')
ylabel('\omega')
legend('ode45', 'RK4');
grid on;

%% Discretize and linearize the dynamic system:
x_lin = zeros(4,1); u_lin = 0;
input.x = x_lin; input.u = u_lin;
output = RK4_integrator( @ode, input );
A = output.sensX;
B = output.sensU;

%% Design of the LQR controller using dlqr:
Q = diag([1 1 1e-1 1e-1]); R = 1e-1;
[K,P] = dlqr(A,B,Q,R);
save lqr.mat A B Q R K P


